; encode wav files generate from Magix Cleaning Lab 2008
; input names of format artist - album_01.wav or album_01.wav
; directory with wav files must contain (only one) anyname.txt with FreeDB title information
; directory with wav files must contain (only one) anyname.jpg with album art file
; check wether it is a Various Artists album if:
;   1. file names contain artist info
;   2. titles read from FreeDB file contain artist info
Opt("MustDeclareVars", 1)

; Constants
Const $mp3Ext		= '.mp3'
Const $wavExt		= '.wav'
Const $txtExt		= '.txt'
Const $jpgExt		= '.jpg'
Const $batExt		= '.bat'
Const $nameTrim		= '_01.wav'
Const $trackCfg		= 'track.cfg'
Const $defToolPath	= 'C:\Program Files\REACT2\tools\'
Const $defCfgPath	= 'C:\Program Files\REACT2\'
Const $defMp3Path	= 'd:\music\mp3'
Const $defFlacPath	= 'd:\music\flac'
Const $noAlbumArt	= 'noAlbumArt'
Const $copyright	= 'A/D conversion by Magix Cleaning Lab 2008'
Const $optFlac		= '-6 -f'
Const $optLameMP3	= '-V4 --noreplaygain --nohist'
Const $verFlac		= '1.2.1b'
Const $verLameMP3	= '3.98.2'
Const $varArtist	= 'Various Artists'

; Variables
Global $trackNum		= 0	; actual track
Global $tracksThisCd	= 0	; total number of tracks
Global $mp3Path			= $defMp3Path
Global $flacPath		= $defFlacPath
Global $guessPath		= ''
Global $toolPath		= ''
Global $filePath		= ''
Global $cfgPath			= ''
Global $fileName		= ''
Global $cmdName			= ''
Global $trackName		= ''
Global $artName			= ''
Global $trackId			= StringToASCIIArray("00")
Global $fileHandle		= -1
Global $processPriority	= 1

Global $artist			[99]
Global $title			[99]
Global $cdArtist		= ''
Global $album			= ''
Global $genre			= ''
Global $year			= 0
Global $isVA			= False

Global $debug			= 0

; get filePath and toolPath from command line
If $CmdLine[0] = 2 Then
	$filePath = $CmdLine[1] & '\'
	$toolPath = $CmdLine[2] & '\'
ElseIf $CmdLine[0] = 1 Then
	$filePath = $CmdLine[1] & '\'
	$toolPath = $defToolPath
; bco - comment Else to test locally
Else
	$help = 'Call format: wavcode <music_path> [<exe_path>]' & @crlf & '-> Example: cdnum "C:\My Music" "C:\Program Files\REACT2\tools"'
	MsgBox(48, 'Help', $help)
	Exit
EndIf
; bco - uncomment next 2 lines to test locally
; $filePath = 'D:\Music\'
; $toolPath = 'D:\Music\REACT_2.0.akku.b04\tools\'

MsgBox(48,'info', 'step 1')

; check if config file exist
If FileExists($defCfgPath & $trackCfg) Then
	$cfgPath = $defCfgPath
ElseIf FileExists($toolPath & $trackCfg) Then
	$cfgPath = $toolPath
ElseIf FileExists($filePath & $trackCfg) Then
	$cfgPath = $filePath
Else
	$cfgPath = FileOpenDialog("Config file for batch encoding", "::{450D8FBA-AD25-11D0-98A8-0800361B1103}", 3, "*.cfg")
	If @error >= 1 Then Exit
EndIf
	
MsgBox(48,'info', 'step 2')

; path to mp3 / flac music files
If Not FileExists($mp3Path) Then
	$mp3Path = FileSelectFolder("Directory to write MP3 music files to", "", 4+1, "::{450D8FBA-AD25-11D0-98A8-0800361B1103}")
	If @error >= 1 Then Exit
	$dirSplit = StringSplit($mp3Path, '\')
	$guessPath = StringTrimRight($mp3Path, StringLen($dirSplit[$dirSplit[0]])) & 'flac'
EndIf
If Not FileExists($flacPath) Then $flacPath = FileSelectFolder("Directory to write FLAC music files to", "", 4+1, $guessPath)
If @error >= 1 Then Exit

; find FreeDB text file and read title information
$fileHandle = FileFindFirstFile($filePath & '*' & $txtExt)
If $fileHandle >= 0 Then
	$fileName = FileFindNextFile($fileHandle)
	If @error Then FileError($filePath, 'FreeDB info text', 0)
	$freeDB = FileRead($fileName, FileGetSize($fileName))
	$dbArr = StringSplit($freeDB, 'TTITLE', 1) ; [0]=# of elements, [1]=everything before 1.TTITLE, [last]=last TTITLE & everything afterwards
	$titleSplit = StringSplit($dbArr[$dbArr[0]], 'EXTD=', 1) ; [last]=last TTITLE trimmed
	$dbArr[$dbArr[0]] = $titleSplit[1]
	$idx = 2
	; separate track name
	While $idx <= $dbArr[0]
		$titleSplit = StringSplit($dbArr[$idx], '=' & @CR, 0)
		;MsgBox(0, 'titleSplit', '0=' & $titleSplit[0] & ' 1=' & $titleSplit[1] & ' 2=' & $titleSplit[2])
		$title[$idx-1] = $titleSplit[2]
		$idx += 1
	WEnd
	$title[0] = $dbArr[0] - 1
	; separate artist from title
	$idx = 1
	While $idx <= $title[0]
		$titleSplit = StringSplit($title[$idx], ' / ', 1)
		If @error Then
			$artist[$idx] = ''
		Else
			$artist[$idx] = $titleSplit[1]
			$title[$idx] = $titleSplit[2]
		EndIf
		$idx += 1
	WEnd
	; get Year and Genre
	$titleSplit = StringSplit($dbArr[1], 'DYEAR=', 1)
	$titleSplit = StringSplit($titleSplit[2], 'DGENRE=', 1)
	$year = StringStripWS($titleSplit[1], 3)
	$genre = StringStripWS($titleSplit[2], 3)
Else
	FileError($filePath, 'FreeDB info text', 0)
EndIf
FileClose($fileHandle)
$tracksThisCd = $dbArr[0]-1

; find album art file
$fileHandle = FileFindFirstFile($filePath & '*' & $jpgExt)
If $fileHandle >= 0 Then
	$artName = FileFindNextFile($fileHandle)
	If @error Then $artName = $noAlbumArt
Else
	$artName = $noAlbumArt
EndIf
FileClose($fileHandle)

; read cfg file into buffer
$cfgData = FileRead($cfgPath & $trackCfg)
If @error Then FileError($filePath, 'Config', 0)

$trackNum = 1
$fileHandle = FileFindFirstFile($filePath & '*' & $wavExt)  

; Loop if the search was successful
If $fileHandle >= 0 Then
	While 1
		$fileName = FileFindNextFile($fileHandle)
		If @error Then ExitLoop

		; check filename and extract artist & album title
		$trArr = StringSplit($fileName, ' - ', 1)
		If @error Then
			If $cdArtist = '' Then
				$cdArtist = InputBox('Album Artist', 'Enter the album artist', $varArtist)
				If @error Then ExitLoop
				$album = StringStripWS($trArr[1], 3)
			EndIf
		ElseIf $trArr[0] = 2 Then
			$trArr[1] = StringStripWS($trArr[1], 3)
			If $artist[$trackNum] = '' Then
				$artist[$trackNum] = $trArr[1]
			ElseIf $artist[$trackNum] <> $trArr[1] Then
				; mismatch between FreeDB info text file and file name
				If MsgBox(49, 'Error', 'Artist info mismatch between FreeDB text file and file name' & @crlf & 'Hit [OK] to keep FreeDB info') = 2 Then ExitLoop
			EndIf
			$album = StringStripWS($trArr[2], 3)
		Else
			MsgBox(48, "Error", "Invalid file name: " & $fileName)
			ExitLoop
		EndIf
		$isVA = $cdArtist = $varArtist
		$album = StringTrimRight($album, StringLen($nameTrim))
		
		; enter artist and title if more tracks found than in db
		If $trackNum > $tracksThisCd Then
			$title[$trackNum] = 'Title' & $trackNum
			If $artist[$trackNum] = '' Then $artist[$trackNum] = $artist[$trackNum-1]
		EndIf
		; output file name
		$trackId[0] = 48 + Int($trackNum / 10)
		$trackId[1] = 48 + Mod($trackNum, 10)
		$trackName = StringFromASCIIArray($trackId) & ' ' & $title[$trackNum]

		; create batch file
		$cmdData = SubstVarsInBatch($cfgData)
		$cmdName = $filePath & $album & '[' & $trackNum & '-' & $tracksThisCd & ']' & $batExt
		If $debug Then
			If MsgBox(49, "Info", $cmdName) = 2 Then ExitLoop
		EndIf
		
		FileDelete($cmdName)
		If FileWrite($cmdName, $cmdData) <= 0 Then FileError($cmdName, 'Track', 1)

		; run batch file
		$pid = Run($cmdName, $filePath, @SW_SHOW)
		Sleep(50) ; allow window to build up
		WinSetTitle("[active]", "", "Encoding Track [" & $trackNum & '-' & $tracksThisCd & "]...")
		ProcessSetPriority($pid, $processPriority)
		ProcessWaitClose($pid)
		
		; increment track numbers
		$trackNum += 1

	WEnd
	; Close the search handle
	FileClose($fileHandle)
EndIf

Func FileError($path, $type, $wr)
	If $wr Then $action = 'written' Else $action = 'found' EndIf
	MsgBox(48, 'Error', $type & ' file not ' & $action & ' in "' & $path & '"')
	Exit
EndFunc

Func SubstVarsInBatch($string)
	$tools = StringTrimRight($toolPath, 1)
	$string = StringReplace($string, "@tools@", $tools)
	$string = StringReplace($string, "@trackname@", $trackName)
	$string = StringReplace($string, "@cover@", $filePath & $artName)
	$string = StringReplace($string, "@VA@", $cdArtist)
	$string = StringReplace($string, "@various@", $isVA)
	$string = StringReplace($string, "@TrackDir_Flac@", $flacPath)
	$string = StringReplace($string, "@TrackDir_MP3@", $mp3Path)
	$string = StringReplace($string, "@Flac@", '1')
	$string = StringReplace($string, "@LameMP3@", '1')
	$string = StringReplace($string, "@ReplayGain@", '1')
	$string = StringReplace($string, "@album@", $album)
	$string = StringReplace($string, "@artist@", $artist[$trackNum])
	$string = StringReplace($string, "@title@", $title[$trackNum])
	$string = StringReplace($string, "@track@", $trackNum)
	$string = StringReplace($string, "@numtracks@", $tracksThisCd)
	$string = StringReplace($string, "@year@", $year)
	$string = StringReplace($string, "@genre@", $genre)
	$string = StringReplace($string, "@comment@", $copyright)
	$string = StringReplace($string, "@Ver_Flac@", $verFlac)
	$string = StringReplace($string, "@Opt_Flac@", $optFlac)
	$string = StringReplace($string, "@Ver_LameMP3@", $verLameMP3)
	$string = StringReplace($string, "@Opt_LameMP3@", $optLameMP3)
	$string = StringReplace($string, "@source@", $filePath & $fileName)
	$string = StringReplace($string, "@Debug@", $debug)
	Return $string
EndFunc
